/*
 * Copyright (c) 2024 Saji Champlin
 */

/*
 * Listens to all can packets and logs them to a file.
 * each power cycle generates a new file increment.
 */


#include <zephyr/storage/disk_access.h>
#include <zephyr/logging/log.h>
#include <zephyr/fs/fs.h>
#include <zephyr/net/canbus.h>
#include <zephyr/init.h>
#include <ff.h>


#define CANLOG_STACKSIZE 1024


#define DISK_DRIVE_NAME "SD"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"
static const char *disk_mount_pt = DISK_MOUNT_PT;

LOG_MODULE_REGISTER(canlog, CONFIG_APP_LOG_LEVEL);

static FATFS fat_fs;
/* mounting info */
static struct fs_mount_t mp = {
	.type = FS_FATFS,
	.fs_data = &fat_fs,
};

int mount_sd() {
    mp.mnt_point = disk_mount_pt;
    int res = fs_mount(&mp);
    if (res != FR_OK) {
        LOG_ERR("SD mount failed: %d", res);
        return -1;
    }
    LOG_INF("init sd ok");
    return 0;
}
SYS_INIT(mount_sd, APPLICATION, 31);

const struct device *const can_dev = DEVICE_DT_GET(DT_CHOSEN(zephyr_canbus));


int init_can() {

    int res;
    if (!device_is_ready(can_dev)) {
		LOG_ERR("CAN %s not ready.\n", can_dev->name);
		return -1;
	}
	res = can_set_mode(can_dev, CAN_MODE_LOOPBACK);
	if (res != 0) {
		printf("Error setting CAN mode [%d]", res);
		return 0;
	}
    res = can_start(can_dev);
    if (res) {
		LOG_ERR("CAN %s failed to start: %d", can_dev->name, res);
        return -1;
    }
    LOG_INF("can start ok");
    return 0;
}

SYS_INIT(init_can, APPLICATION, 32);


/*
 * This init is specifically to setup the filters before we start
 * the routine.
 * 
 */

int setup_can_filters() {
#ifndef CONFIG_LOOPBACK_MODE
	int ret = can_set_mode(can_dev, CAN_MODE_LOOPBACK);
	if (ret != 0) {
		printf("Error setting CAN mode [%d]", ret);
		return 0;
	}
#endif

}

const struct can_filter all_packets_filter =  {
    .flags = 0U,
    .id = 0,
    .mask = 0,
};
const struct can_filter high_prio_filter = {
    .flags = 0U,
    .id = 0U,
    .mask = 0xE00U,
};



K_MSGQ_DEFINE(sd_canframes, sizeof(struct can_frame), 20, 1);
void can_task() {
    struct fs_file_t logfile;
    int res;
    struct can_frame incoming_frame;
    
    fs_file_t_init(&logfile);
    res = fs_open(&logfile, "/SD:/log3.txt", FS_O_CREATE |FS_O_APPEND | FS_O_WRITE);
    if (res != FR_OK) {
        LOG_ERR("Open file failed: %d", res);
        return;
    }
    res = can_add_rx_filter_msgq(can_dev, &sd_canframes, &all_packets_filter);
    if (res) {
        LOG_ERR("Could not add filter: %d", res);
        return;
    }
    while (1) {
        // read the frame.
        k_msgq_get(&sd_canframes, &incoming_frame, K_FOREVER);
        fs_write(&logfile, &incoming_frame, sizeof(incoming_frame));
        fs_sync(&logfile);
        LOG_INF("write!");
    }
}


/*
* work queue handler for SD card log.
*/


K_THREAD_DEFINE(canlog_th, CANLOG_STACKSIZE, can_task, NULL, NULL, NULL, 5,  0, 0);
// create the task, mount the FS, listen to the can bus.
